#!/bin/bash

############################################################################################################################
#                                                                                                                          #
#                                                  INESC-ID/L2F                                                            #
#                               Instituto de Engenharia de Sistemas e Computadores                                         #
#                                    Investigaco e Desenvolvimento em Lisboa,                                              #
#                                 Laboratario de Sistemas de Lingua Falada (L2F)                                           #
#                                                                                                                          #
#                                                                                                                          #
############################################################################################################################
#                                                                                                                          #
#                                                                                                                          #
# Authors: Alberto Abad     (alberto@l2f.inesc-id.pt)                                                                      #
#                                                                                                                          #
# Date   : 11 Oct 2013                                                                                                     #
#                                                                                                                          #
# Version: 1.0                                                                                                             #
#                                                                                                                          #
############################################################################################################################
#
#  Usage: PrepareForFusion.sh -q <tlistxml> -r <rttm> -o <outputfile> [opts] <stdlistfile1> [stdlistfile2] [stdlistfile3]  ... [stdlistfileN] 
#
#     <stdlistfile*> input score stdlist file in the SWS2012 format (*.stdlist.xml)        | - Required argument (at least 1)
#     -q <tlistxml> termlist file in the SWS2012 format (*.tlist.xml)                      | - Required parameter
#     -r <rttm> rttm file in the SWS2012 format (*.rttm)                                   | - Required parameter
#     -o <outputfile> output file name                                                     | - Required parameter
#     -z <value> score z-norm type (none, qnorm, fnorm, qfnorm, fqnorm)                    | - Default: none
#     -g add ground-truth information to the outputfile
#     -t <filename> saves the number of true terms in the reference per query (implies -g)
#     -m <value> apply majority voting fusion with <value> minimum number of votes         | - Default: 1
#     -n <value> method for creating default scores (0: average of the other detections (MV approach); 1; min per query, 2: global min, 3: histogram based)  | - Default: 0
#     -d debug mode, done remove auxiliar files stored in /tmp/tmpdir.$$
#     -h help 
#
#     NOTE: Requires Matlab (or octave) and perl. It also depends on the perl scripts align_score_files.pl, heuristicScoring.pl and create_groundtruth_centerdistance.pl 
#           that should be located in the same folder of the main script
#

MATLAB_BIN="/afs/l2f/usr/local/MATLAB/R2010b/bin/matlab"

############################################################################################################################
#                                                                                                                          #
# qnorm()- this function applies per query normalization to an stdlist.xml results file                                    #
#                                                                                                                          #
###########################################################################################################################

function qnorm {

  std=$1 
  stdnorm=$2

  mkdir -p /tmp/tmpdir.$$/

  awk 'BEGIN{first=1;}{if (/termid=/) {if (first==1) printf $2" "; else printf "\n"$2" "; first=0;}; if (/score=/) printf $6" ";}END{print "";}' $std  | sed -e "s,score=,,g" | tr -d '"'  | cut -d' ' -f2- > /tmp/tmpdir.$$/scoresQ.txt


  echo "function apply_qnorm" > /tmp/tmpdir.$$/apply_qnorm.m
  echo "fr=fopen('scoresQ.txt','r');" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "fw=fopen('scoresNORMQ.txt','w');" >> /tmp/tmpdir.$$/apply_qnorm.m

  echo "while (1)" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "sc=fgetl(fr);" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "if (sc==-1), break; end" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "v=sscanf(sc,'%f',Inf);" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "if (isempty(v)), continue; end"  >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "m = mean(v);" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "s = std(v,1);" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "if (s==0), s=1;end" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "v_norm = (v -m)./s;" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "fprintf(fw,'%f ',v_norm);" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "end" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "fprintf(fw,'\n');" >> /tmp/tmpdir.$$/apply_qnorm.m
  echo "fclose(fr);" >>  /tmp/tmpdir.$$/apply_qnorm.m
  echo "fclose(fw);" >>  /tmp/tmpdir.$$/apply_qnorm.m
  echo "end" >>  /tmp/tmpdir.$$/apply_qnorm.m

  currentDir=`pwd`
  cd /tmp/tmpdir.$$/
  #   octave --eval apply_qnorm >& /dev/null
  $MATLAB_BIN -nosplash -nodisplay -nojvm -r apply_qnorm 1> /dev/null < /dev/null
  cd $currentDir

  cat /tmp/tmpdir.$$/scoresNORMQ.txt $std | \
  awk 'BEGIN{nScore=1;}{if (NR==1) {for (n=1;n<=NF;n++) sc[n]=$n;} else { if (/score=/) {print $1,$2,$3,$4,$5, "score=\""sc[nScore]"\"", $7;nScore++;} else {print $0}}}'  > $stdnorm


}

############################################################################################################################
#                                                                                                                          #
# fnorm()- this function applies per file normalization to an stdlist.xml results file                                    #
#                                                                                                                          #
############################################################################################################################

function fnorm {

  std=$1 
  stdnorm=$2

  mkdir -p /tmp/tmpdir.$$/

  grep "score=" $std  | cut -d\" -f2,10 | tr -s '"' ' '  | awk '{print NR,$0}' | sort -k2 > /tmp/tmpdir.$$/scoresF.tmp

  cut -d' ' -f1 /tmp/tmpdir.$$/scoresF.tmp > /tmp/tmpdir.$$/scoresF.order

  cut -d' ' -f2- /tmp/tmpdir.$$/scoresF.tmp  | awk 'BEGIN{first=1;}{if (first==1) {printf $0" "; ant=$1;first=0} else {if ($1==ant) printf $2" "; else {printf "\n"$0" "; ant=$1;}} }END{print "";}' | cut -d' ' -f2- > /tmp/tmpdir.$$/scoresF.txt


  echo "function apply_fnorm" >  /tmp/tmpdir.$$/apply_fnorm.m
  echo "fr=fopen('scoresF.txt','r');" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "fw=fopen('scoresNORMF.txt','w');" >> /tmp/tmpdir.$$/apply_fnorm.m

  echo "while (1)" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "sc=fgetl(fr);" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "if (sc==-1), break; end" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "v=sscanf(sc,'%f',Inf);" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "m = mean(v);" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "s = std(v,1);" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "if (s==0), v_norm = (v -m);" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "else, v_norm = (v -m)./s; end" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo -n "fprintf(fw,'%f\n',v_norm);" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "end" >> /tmp/tmpdir.$$/apply_fnorm.m
  echo "fclose(fr);" >>  /tmp/tmpdir.$$/apply_fnorm.m
  echo "fclose(fw);" >>  /tmp/tmpdir.$$/apply_fnorm.m
  echo "end" >>  /tmp/tmpdir.$$/apply_fnorm.m

  currentDir=`pwd`
  cd /tmp/tmpdir.$$/
  #   octave --eval apply_fnorm >& /dev/null
  $MATLAB_BIN -nosplash -nodisplay -nojvm -r apply_fnorm 1> /dev/null < /dev/null
  cd $currentDir

  paste -d' ' /tmp/tmpdir.$$/scoresF.order /tmp/tmpdir.$$/scoresNORMF.txt | sort -k1 -n | cut -d' ' -f2 | tr -s '\n' ' ' | awk '{print $0}' > /tmp/tmpdir.$$/scoresNORMFsorted.txt

  cat /tmp/tmpdir.$$/scoresNORMFsorted.txt $std | \
  awk 'BEGIN{nScore=1;}{if (NR==1) {for (n=1;n<=NF;n++) sc[n]=$n;} else { if (/score=/) {print $1,$2,$3,$4,$5, "score=\""sc[nScore]"\"", $7;nScore++;} else {print $0}}}'  > $stdnorm


}

function qfnorm(){

 mkdir -p /tmp/tmpdir.$$/
 qnorm $1 /tmp/tmpdir.$$/qfnorm.qnorm.txt
 fnorm /tmp/tmpdir.$$/qfnorm.qnorm.txt $2 

}


function fqnorm(){

 mkdir -p /tmp/tmpdir.$$/
 fnorm $1 /tmp/tmpdir.$$/$2.fqnorm.qnorm.txt
 fnorm /tmp/tmpdir.$$/$2.fqnorm.qnorm.txt $2 
}


function none(){
 cp $1 $2
}

function fromsws2raw(){
  awk -F\" '{if ((/termid/)) term=$2; else if (/term file/) printf("%s %s %.2f %.2f %.4f\n",term, $2,$6,$8,$10)}' $1 > $2
}


function showhelp(){
tail -n+21 $0 | head -n18  | cut -c2-
if [ $debug -eq "0" ]; then
    rm -r  /tmp/tmpdir.$$/
fi
exit 
}

  
## PARSE OPTIONS!!
   
  mkdir -p /tmp/tmpdir.$$/

  outFile=""
  znorm="none"
  debug=0
  gt=0
  minVotes=1 ## This is equivalent to keeping all of the candidates
  defScoreMethod=0
  MAXCENTERDISTANCE="2.0" # This variable controls the maximum allowed distance between centers for hypothesis vs groundtruth alignment


  instalDir=`dirname $0`


  while getopts ghdz:f:o:q:r:m:n:t: opt
  do
    case "$opt" in
      n) defScoreMethod="$OPTARG";
	  if [ $defScoreMethod -ne "0" ] && [ $defScoreMethod -ne "1" ] && [ $defScoreMethod -ne "2" ] && [ $defScoreMethod -ne "3" ]; then
	    echo "Argument $defScoreMethod is an invalid value for paramer -n"
	    exit;
	  fi;;
      m) minVotes="$OPTARG";
	  if [ $minVotes -le "0" ] ; then
	    echo "Argument $minVotes is an invalid value for paramer -m"
	    exit;
	  fi;;
      z) znorm="$OPTARG";
	 if [ $znorm != "none" ] && [ $znorm != "qnorm" ] && [ $znorm != "fnorm" ] && [ $znorm != "qnorm" ] && [ $znorm != "fqnorm" ] && [ $znorm != "qfnorm" ]; then
	    echo "Argument $znorm is an invalid value for paramer -z"
	    exit;
	 fi;;
      o) outFile="$OPTARG";;
      q) queryFile="$OPTARG";
	if [ ! -f $queryFile ]; then
	    echo "  File: $queryFile" 
	    echo "  Does not exist. For usage help run $0 -h" 
	    exit
 	elif  [ `expr $queryFile : .*.tlist.xml` -lt 1 ];then
	  echo
	  echo "The argument must be a .tlist.xml file. For usage help run $0 -h" 
	  echo
	  exit
	fi
	querylist=/tmp/tmpdir.$$/querycollection.list
	grep termid $queryFile  | cut -d\" -f2 > $querylist;;
      r) rttm="$OPTARG";
	if [ ! -f $rttm ]; then
	    echo "  File: $rttm" 
	    echo "  Does not exist. For usage help run $0 -h" 
	    exit
 	elif  [ `expr $rttm : .*.rttm` -lt 1 ];then
	  echo
	  echo "The argument must be a .rttm file. For usage help run $0 -h" 
	  echo
	  exit
	fi
	datalist=/tmp/tmpdir.$$/datacollection.list
	cut -d' ' -f2 $rttm  | sort | uniq > $datalist;;
      g) gt=1;;
      t) refQueriesFile="$OPTARG"
	 gt=1;;
      d) debug=1;;
      h) showhelp;;
      *) showhelp;;
    esac
  done

  if [ -z "$outFile" ]; then
    showhelp;
  fi

  if [ -z "$rttm" ]; then
    showhelp;
  fi

  if [ -z "$queryFile" ]; then
    showhelp;
  fi
  shift $((OPTIND-1))



if [ $# -lt 1 ]
then
    showhelp;
else

  numSystems=0
  for arg in "$@"
  do
       if [ ! -f $arg ]; then
		echo "File $arg does not exist"
		exit;
       fi

      let "numSystems+=1"
      fileSystem[$numSystems]=$arg; 
  done             

  ## minVotes parameter must be between 1 and nSystems
  if [ $minVotes -gt "$numSystems" ] ; then
      echo "Argument $minVotes is an invalid value for paramer -m"
      exit;
  fi


  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #   
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #    
  # #   FIRST STEP - APPLY SCORE NORM                               # # 
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #    
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #   

  ## Apply score normalization to each file and convert to raw format
  for n in `seq 1 $numSystems`; do
    echo ${fileSystem[$n]}

    $znorm ${fileSystem[$n]} /tmp/tmpdir.$$/scores.system$n.stdlist.xml
    fromsws2raw /tmp/tmpdir.$$/scores.system$n.stdlist.xml /tmp/tmpdir.$$/scores.system$n.txt
    cp /tmp/tmpdir.$$/scores.system$n.txt /tmp/tmpdir.$$/scores.system$n.ori
  done



  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #   
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
  # #   SECOND STEP - ALIGN DETECTIONS OF MULTIPLE SYSTEMS          # # 
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

  ## Parallelize/align multiple systems detection candidates
  if [ $numSystems -lt 2 ]; then
      n=1
      #       I also do the alignment when I have just one seem to assure that I have exactly the same sorting than in the groundtruth
      $instalDir/align_score_files.pl /tmp/tmpdir.$$/scores.system$n.txt /tmp/tmpdir.$$/scores.system$n.txt $querylist $datalist | cut -d' ' -f1-5  > /tmp/tmpdir.$$/scores.ALL_SYSTEMS.txt
  else
    for j in `seq 2 $numSystems`; do
      let "j1=j-1"
      for i in `seq 1 $j1`; do
	 $instalDir/align_score_files.pl /tmp/tmpdir.$$/scores.system$i.txt /tmp/tmpdir.$$/scores.system$j.txt $querylist $datalist > /tmp/tmpdir.$$/scores.system$i-$j.MERGED.txt
	 cut -d' ' -f1-5 /tmp/tmpdir.$$/scores.system$i-$j.MERGED.txt > /tmp/tmpdir.$$/scores.system$i.txt
      done
      cut -d' ' -f1-4,6 /tmp/tmpdir.$$/scores.system$i-$j.MERGED.txt > /tmp/tmpdir.$$/scores.system$j.txt
    done
  
    cp /tmp/tmpdir.$$/scores.system1.txt /tmp/tmpdir.$$/fuse.results.txt
    for j in `seq 2 $numSystems`; do
      cut -d' ' -f5 /tmp/tmpdir.$$/scores.system$j.txt | paste -d' ' /tmp/tmpdir.$$/fuse.results.txt - > /tmp/tmpdir.$$/fuse.results.txt.tmp
      mv /tmp/tmpdir.$$/fuse.results.txt.tmp /tmp/tmpdir.$$/fuse.results.txt
    done
    cp /tmp/tmpdir.$$/fuse.results.txt /tmp/tmpdir.$$/scores.ALL_SYSTEMS.txt
  fi


  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #   
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
  # #   THIRD STEP - MAJORITY VOTING FILTERING                      # # 
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

  ## Keep only candidates with minimum -m <value> detections
  ## Apply Majority Voting filtering 

  cp /tmp/tmpdir.$$/scores.ALL_SYSTEMS.txt /tmp/tmpdir.$$/scores.ALL_SYSTEMS.beforeMV.txt

  cat  /tmp/tmpdir.$$/scores.ALL_SYSTEMS.beforeMV.txt | awk 'BEGIN{nSys="'$numSystems'";MV="'$minVotes'"}{det=0;for (n=5;n<=4+nSys;n++) {if ($n!="NOT_DETECTED") {det=det+1;}} if (det>=MV) {print $0;}}' > /tmp/tmpdir.$$/scores.ALL_SYSTEMS.txt


  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #   
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
  # #   FORTH STEP - CREATE SCORES FOR NOT DETECTED CANDIDATES      # # 
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

  ## APPLY HEURISTICS TO ASSFIGN NEW SCORES ACCORDING TO -n flag
  cp /tmp/tmpdir.$$/scores.ALL_SYSTEMS.txt /tmp/tmpdir.$$/scores.ALL_SYSTEMS.beforeHS.txt
  $instalDir/heuristicScoring.pl /tmp/tmpdir.$$/scores.ALL_SYSTEMS.beforeHS.txt $numSystems $defScoreMethod $querylist  > /tmp/tmpdir.$$/scores.ALL_SYSTEMS.txt


  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #   
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
  # #   FIFTH STEP - CREATE AND CONCATENATE GROUND-TRUTH            # # 
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
  # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

  ## Add ground-truth info
  if [ $gt -eq "1" ]; then
    echo "Adding ground-truth information to output file"

    querycontent=/tmp/tmpdir.$$/querycontent.list
    grep termid $queryFile | cut -d'<' -f3 | cut -d'>' -f2  | awk '{print tolower($0)}'> $querycontent
    paste -d! $querycontent $querylist | awk -F! '{print "s,!"$1"!,"$2",g"}' > /tmp/tmpdir.$$/sed.file

  
    NW=`awk 'BEGIN{MNF=0;}{if (NF>MNF) MNF=NF;}END{print MNF}' $querycontent`

    grep lex $rttm | awk '{{print $2, $2,$4, $5, tolower($6);}}' > /tmp/tmpdir.$$/lex1
    nl=`wc -l /tmp/tmpdir.$$/lex1 | cut -d' ' -f1`

    let NW--
    for ng in `seq 1 $NW`; do
      let nl--
      nng=`expr $ng + 1`
      tail -n $nl /tmp/tmpdir.$$/lex1 | paste -d' ' /tmp/tmpdir.$$/lex${ng} - | awk '{{i=$3; f=$8+$9; d=f-i; print $1,$7,i,d,$5"*"$10}}' > /tmp/tmpdir.$$/lex${nng}
    done

    let NW++
    for ng in `seq 1 $NW`; do
      
      awk '{if ($1==$2) print $2,$3,$4,$5;}' /tmp/tmpdir.$$/lex${ng}  > /tmp/tmpdir.$$/lex${ng}.tmp 
      
      cut -d' ' -f1-3 /tmp/tmpdir.$$/lex${ng}.tmp  > /tmp/tmpdir.$$/lex${ng}.tmp2
      cut -d' ' -f4 /tmp/tmpdir.$$/lex${ng}.tmp | tr -s '*' ' ' | awk '{print "!"$0"!"}' | paste -d' ' /tmp/tmpdir.$$/lex${ng}.tmp2 - > /tmp/tmpdir.$$/lex${ng}
     
    done
    rm  /tmp/tmpdir.$$/lex*.tmp*


    sed -f /tmp/tmpdir.$$/sed.file /tmp/tmpdir.$$/lex* | grep -v "\!" | awk '{print $4,$1,$2,$3}' > /tmp/tmpdir.$$/groundtruth
    cut -d' ' -f1-4 /tmp/tmpdir.$$/scores.ALL_SYSTEMS.txt > /tmp/tmpdir.$$/candidates

    # I constrast to the previous alignments (that perform the alignment based on overlaping candidates) for the groundtruth we use the same rule as for the evaluation (maximum istance betweeen centers of hypothesis and groundtruths)
    # The last parameter that takes this script is the maximum allowed distance between centers in secomds. 
    # By default MAXCENTERDISTANCE is set to 2.0 like the official SWS2013 evaluation
    $instalDir/create_groundtruth_centerdistance.pl /tmp/tmpdir.$$/candidates /tmp/tmpdir.$$/groundtruth $querylist $datalist $MAXCENTERDISTANCE > /tmp/tmpdir.$$/complete_alignment.txt
    grep -v ONLY_IN_THE_REF /tmp/tmpdir.$$/complete_alignment.txt | cut -d' ' -f5 > /tmp/tmpdir.$$/aligned_groundtruth.txt
    paste -d' ' /tmp/tmpdir.$$/scores.ALL_SYSTEMS.txt /tmp/tmpdir.$$/aligned_groundtruth.txt >  $outFile

    if [ ! -z $refQueriesFile ]; then
    
      awk '{if (($NF==1)||(/ONLY/)) print $0}'  /tmp/tmpdir.$$/complete_alignment.txt >  /tmp/tmpdir.$$/all_targets.txt
      for q in `cat $querylist`; do
	echo "$q `cut -d' ' -f1 /tmp/tmpdir.$$/all_targets.txt | grep -c $q`" 
      done > $refQueriesFile
    fi
    
  else
    awk '{print $0,"0";}' /tmp/tmpdir.$$/scores.ALL_SYSTEMS.txt > $outFile
  fi

  if [ $debug -eq "0" ]; then
    rm -r  /tmp/tmpdir.$$/
  fi
  
fi



