#!/usr/bin/perl 

############################################################################################################################
#                                                                                                                          #
#                                                  INESC-ID/L2F                                                            #
#                               Instituto de Engenharia de Sistemas e Computadores                                         #
#                                    Investigaco e Desenvolvimento em Lisboa,                                              #
#                                 Laboratario de Sistemas de Lingua Falada (L2F)                                           #
#                                                                                                                          #
#                                                                                                                          #
############################################################################################################################
#                                                                                                                          #
#                                                                                                                          #
# Authors: Alberto Abad     (alberto@l2f.inesc-id.pt)                                                                      #
#                                                                                                                          #
# Date   : 11 Oct 2013                                                                                                     #
#                                                                                                                          #
# Version: 1.0                                                                                                             #
#                                                                                                                          #
############################################################################################################################
#                                                                                                                          
#  This perl script is used by the PrepareForFusion.sh script to align the detection candidates given by two heterogenepus 
#  systems.                                                                                                                
#                                                                                                                          
#  It takes as input the detection candidates (in an internal specific format) and a list of all queries and data       
#  files of the collection.                                                                                                
#                                                                                                                          
#  The ALIGNMENT is based on candidate overlapping. Any two candidate detections (of the same query in the same file) that 
#  overlap are considered aligned. This criteria is different to the one of the evaluation (which is based on maximum      
#  allowed distance between the center of candidates). There is not any particular reason for applying one or the other    
#  and results should be quite independent on this.                                                                        
#                                                                                                                          
#  Notice that some of the comments inside the script are a  mixture of English, Spanish and Portuguese                     
#  Sorry for that :-)                                                                                                      
#                                                                                                                          
#                                                                                                                          
############################################################################################################################

my $nArgs = $#ARGV +1;

if (($nArgs!=4)) {die "Invalid number of arguments $nArgs";}

my $system1 = $ARGV[0];
my $system2 = $ARGV[1];
my $querylist = $ARGV[2];
my $datalist = $ARGV[3];

                                                                                                                                                                                                                                                                     
my %mapq = ();                                                                                                                                                                                                                                                                  
my %mapd = ();                                                                                                                                                                                                                                                                  


## Read list of queries
                                                                                                                                                                                                                                                                               
open QUERIES, $querylist or die "could not open queries list file $querylist";                            
$value=1;                                                                                                                                                                              
while (<QUERIES>) {                                                                                                                                                                                                                                                            
chomp;                                                                                                                                                                                                                                                                         
$label=$_;
$mapq{$label} = $value;
$max_num_queries = $value;
$QUERY[$value]=$label;
$value=$value+1;
}
close(QUERIES);

## Read list of files of the data collection
open DATA, $datalist or die "could not open data collection list file $datalist";
$value=1;                                                                                                                                                                              
while (<DATA>) {                                                                                                                                                                                                                                                            
chomp;                                                                                                                                                                                                                                                                         
$label=$_;
$mapd{$label} = $value;
$max_num_files = $value;
$FILE[$value]=$label;
$value=$value+1;
}
close(DATA);

for ($q=1;$q<=$max_num_queries;$q++) {
  for ($f=1;$f<=$max_num_files;$f++) {
    $NUM_S1[$q][$f] = 0;
    $NUM_S2[$q][$f] = 0;
  }
}



## Store all detections of system1
open SYSTEM1, $system1 or die "could not open file of scores of system $system1";
while (<SYSTEM1>) {
  chomp;

  @line = split(/ /, $_);
   
    $query = $line[0];
    $file = $line[1];
    $bt = $line[2];
    $dur = $line[3];
    $sc = $line[4];
    $et = $bt + $dur;
    
    
    $NUM_S1[$mapq{$query}][$mapd{$file}] = $NUM_S1[$mapq{$query}][$mapd{$file}] + 1; ## Counts the numbers of detections of the same query in a certain file
    
    $idx = $NUM_S1[$mapq{$query}][$mapd{$file}];
    $BEGINTIME_S1[$mapq{$query}][$mapd{$file}][$idx] = $bt; ##Store TIMING and SCORE info
    $DURATION_S1[$mapq{$query}][$mapd{$file}][$idx] = $dur;
    $END_S1[$mapq{$query}][$mapd{$file}][$idx] = $et;
    $SCORE_S1[$mapq{$query}][$mapd{$file}][$idx] = $sc;

}

close(SYSTEM1);

## Store all detections of system2
open SYSTEM2, $system2 or die "could not open file of scores of system  $system2";
while (<SYSTEM2>) {
  chomp;

  @line = split(/ /, $_);
   
    $query = $line[0];
    $file = $line[1];
    $bt = $line[2];
    $dur = $line[3];
    $sc = $line[4];
    $et = $bt + $dur;
    
    
    $NUM_S2[$mapq{$query}][$mapd{$file}] = $NUM_S2[$mapq{$query}][$mapd{$file}] + 1; ## Counts the numbers of detections of the same query in a certain file
    
    $idx = $NUM_S2[$mapq{$query}][$mapd{$file}];
    $BEGINTIME_S2[$mapq{$query}][$mapd{$file}][$idx] = $bt; ##Store TIMING and SCORE info
    $DURATION_S2[$mapq{$query}][$mapd{$file}][$idx] = $dur;
    $END_S2[$mapq{$query}][$mapd{$file}][$idx] = $et;
    $SCORE_S2[$mapq{$query}][$mapd{$file}][$idx] = $sc;

   
}

close(SYSTEM2);



for ($q=1;$q<=$max_num_queries;$q++) {
    $DEF_SCORE_S1[$q]="NOT_DETECTED";
    $DEF_SCORE_S2[$q]="NOT_DETECTED";
}  


## Do the merge
for ($q=1;$q<=$max_num_queries;$q++) {
  for ($f=1;$f<=$max_num_files;$f++) {
    
    if ($NUM_S1[$q][$f] > 0) {
          if ($NUM_S2[$q][$f] > 0) { ##Queries detected by both systems
	    ## Detect overlaps
	    my $OVERLAP;
	    for ($n=1;$n<=$NUM_S1[$q][$f];$n++) {
		for ($m=1;$m<=$NUM_S2[$q][$f];$m++) {
		  $OVERLAP[$n][$m] = 0;
		  $OVERLAP_S1[$n] = 0;
		  $OVERLAP_S2[$m] = 0; ## Para asegurar que un label del sistema 2 es asociada a 1 sola label del 1
	    }
	    }
	    for ($n=1;$n<=$NUM_S1[$q][$f];$n++) {
		for ($m=1;$m<=$NUM_S2[$q][$f];$m++) {
		  if ($BEGINTIME_S1[$q][$f][$n] <= $BEGINTIME_S2[$q][$f][$m]) { ## A query do sistema1 comena antes que a do sistema2
		      if ($END_S1[$q][$f][$n] > $BEGINTIME_S2[$q][$f][$m]) { ## A query do sistema1 acaba depois do inicio da do sistema2 -> overlapping
		  
			  if ($OVERLAP_S2[$m] == 0) { ### I can only associate 1 segment of one of the files to 1 segment of teh other, it is biyective
			    $OVERLAP[$n][$m] = 1;
			    $OVERLAP_S1[$n] = $m;
			    $OVERLAP_S2[$m] = $n; 
			    last; ## Cada label do sistema1 s poder overlapear com 1 label do sistema2
			  }
		      }
		  }
		  else { #A query do sistema 1 comena depois que a do sistema 2
		      if ($BEGINTIME_S1[$q][$f][$n] < $END_S2[$q][$f][$m]) { ## A query do sistema2 comea antes do fim da do sistema2 -> overlapping
			  if ($OVERLAP_S2[$m] == 0) { ### I can only associate 1 segment of one of the files to 1 segment of teh other, it is biyective
			    $OVERLAP[$n][$m] = 1;
			    $OVERLAP_S1[$n] = $m;
			    $OVERLAP_S2[$m] = $n; 
			    last; ## Cada label do sistema1 s poder overlapear com 1 label do sistema2
			  }
		      }
		  }
		}
	    }

	  ## Print not overlaped candidates of system1 and the overlapped ones
	  for ($n=1;$n<=$NUM_S1[$q][$f];$n++) {
		if ($OVERLAP_S1[$n]!=0) {
		      $overlap=$OVERLAP_S1[$n];
		      if($BEGINTIME_S1[$q][$f][$n]>$BEGINTIME_S2[$q][$f][$overlap]) {$BEGINTIME_S1[$q][$f][$n]=$BEGINTIME_S2[$q][$f][$overlap];}
		      if($END_S1[$q][$f][$n]<$END_S2[$q][$f][$overlap]) {$END_S1[$q][$f][$n]=$END_S2[$q][$f][$overlap];}
		      
		      $DURATION_S1[$q][$f][$n] =  $END_S1[$q][$f][$n] - $BEGINTIME_S1[$q][$f][$n];
		      printf("%s %s %.2f %.2f ", $QUERY[$q], $FILE[$f], $BEGINTIME_S1[$q][$f][$n], $DURATION_S1[$q][$f][$n]);
		      print "$SCORE_S1[$q][$f][$n] $SCORE_S2[$q][$f][$overlap]\n";
		  }
		else {
			print "$QUERY[$q] $FILE[$f] $BEGINTIME_S1[$q][$f][$n] $DURATION_S1[$q][$f][$n] $SCORE_S1[$q][$f][$n] $DEF_SCORE_S2[$q]\n";
		}
	    }
	
	  ## Print remaining candidates of system2 (not overlapped)
	  for ($m=1;$m<=$NUM_S2[$q][$f];$m++) {
	      if ($OVERLAP_S2[$m]==0) {
		      print "$QUERY[$q] $FILE[$f] $BEGINTIME_S2[$q][$f][$m] $DURATION_S2[$q][$f][$m] $DEF_SCORE_S1[$q] $SCORE_S2[$q][$f][$m]\n";
		  }
	  }
	  

	  }
	  else { ##Query detected only by system1
	    for ($n=1;$n<=$NUM_S1[$q][$f];$n++) {
		  print "$QUERY[$q] $FILE[$f] $BEGINTIME_S1[$q][$f][$n] $DURATION_S1[$q][$f][$n] $SCORE_S1[$q][$f][$n] $DEF_SCORE_S2[$q]\n"; 
	    }
	  }

    }
    elsif ($NUM_S2[$q][$f] > 0) { ##Only detected by system2
	    for ($n=1;$n<=$NUM_S2[$q][$f];$n++) {
		  print "$QUERY[$q] $FILE[$f] $BEGINTIME_S2[$q][$f][$n] $DURATION_S2[$q][$f][$n] $DEF_SCORE_S1[$q] $SCORE_S2[$q][$f][$n]\n";
	    }
    }
  }
}

