#!/usr/bin/perl 

############################################################################################################################
#                                                                                                                          #
#                                                  INESC-ID/L2F                                                            #
#                               Instituto de Engenharia de Sistemas e Computadores                                         #
#                                    Investigaco e Desenvolvimento em Lisboa,                                              #
#                                 Laboratario de Sistemas de Lingua Falada (L2F)                                           #
#                                                                                                                          #
#                                                                                                                          #
############################################################################################################################
#                                                                                                                          #
#                                                                                                                          #
# Authors: Alberto Abad     (alberto@l2f.inesc-id.pt)                                                                      #
#                                                                                                                          #
# Date   : 11 Oct 2013                                                                                                     #
#                                                                                                                          #
# Version: 1.0                                                                                                             #
#                                                                                                                          #
############################################################################################################################
#
#  This perl script is used by the PrepareForFusion.sh script to obtain an aligned groundtruth for the detected 
#  candidates.
#  
#  It takes as input the aligned detection candidate scores, the reference file, the list of all queries, the list of all
#  data collection files and a maximum allowed distance in seconds for aligning an hypotehesis and reference label
#
#  The ALIGNMENT is based on maximum allowed distance between the center of hypotehesis and reference labels 
#     
###########################################################################################################################

my $system1 = $ARGV[0];
my $system2 = $ARGV[1]; ## In this case system2 is the reference
my $querylist = $ARGV[2];
my $datalist = $ARGV[3];
my $MAXDISTANCE = $ARGV[4];                                                                                                                                                                                                                                                         
                                                 
my %mapq = ();                                                                                                                                                                                                                                                                  
my %mapd = ();                                                                                                                                                                                                                                                                  


## Read list of queries
                                                                                                                                                                                                                                                                               
open QUERIES, $querylist or die "could not open query list file $querylist";                            
$value=1;                                                                                                                                                                              
while (<QUERIES>) {                                                                                                                                                                                                                                                            
chomp;                                                                                                                                                                                                                                                                         
$label=$_;
$mapq{$label} = $value;
$max_num_queries = $value;
$QUERY[$value]=$label;
$value=$value+1;
}
close(QUERIES);

## Read list of files of the data collection
open DATA, $datalist or die "could not open data collection list file $datalist";
$value=1;                                                                                                                                                                              
while (<DATA>) {                                                                                                                                                                                                                                                            
chomp;                                                                                                                                                                                                                                                                         
$label=$_;
$mapd{$label} = $value;
$max_num_files = $value;
$FILE[$value]=$label;
$value=$value+1;
}
close(DATA);

for ($q=1;$q<=$max_num_queries;$q++) {
  for ($f=1;$f<=$max_num_files;$f++) {
    $NUM_S1[$q][$f] = 0;
    $NUM_S2[$q][$f] = 0;
  }
}

## Store all detections of system1
open SYSTEM1, $system1 or die "could not open the score file $system1";
while (<SYSTEM1>) {
  chomp;

  @line = split(/ /, $_);
   
    $query = $line[0];
    $file = $line[1];
    $bt = $line[2];
    $dur = $line[3];
    $et = $bt + $dur;
    
    
    $NUM_S1[$mapq{$query}][$mapd{$file}] = $NUM_S1[$mapq{$query}][$mapd{$file}] + 1; ## Counts the numbers of detections of the same query in a certain file
    
    $idx = $NUM_S1[$mapq{$query}][$mapd{$file}];
    $BEGINTIME_S1[$mapq{$query}][$mapd{$file}][$idx] = $bt; ##Store TIMING and SCORE info
    $DURATION_S1[$mapq{$query}][$mapd{$file}][$idx] = $dur;
    $END_S1[$mapq{$query}][$mapd{$file}][$idx] = $et;
    $CENTER_S1[$mapq{$query}][$mapd{$file}][$idx] = ($bt+$et)/2;

}

close(SYSTEM1);

## Store all detections of system2
open SYSTEM2, $system2 or die "could not open the reference file $system2";
while (<SYSTEM2>) {
  chomp;

  @line = split(/ /, $_);
   
    $query = $line[0];
    $file = $line[1];
    $bt = $line[2];
    $dur = $line[3];
    $et = $bt + $dur;
    
    
    $NUM_S2[$mapq{$query}][$mapd{$file}] = $NUM_S2[$mapq{$query}][$mapd{$file}] + 1; ## Counts the numbers of detections of the same query in a certain file
    
    $idx = $NUM_S2[$mapq{$query}][$mapd{$file}];
    $BEGINTIME_S2[$mapq{$query}][$mapd{$file}][$idx] = $bt; ##Store TIMING and SCORE info
    $DURATION_S2[$mapq{$query}][$mapd{$file}][$idx] = $dur;
    $END_S2[$mapq{$query}][$mapd{$file}][$idx] = $et;
    $CENTER_S2[$mapq{$query}][$mapd{$file}][$idx] = ($bt+$et)/2;

}

close(SYSTEM2);

## Do the merge
for ($q=1;$q<=$max_num_queries;$q++) {
  for ($f=1;$f<=$max_num_files;$f++) {
    
    if ($NUM_S1[$q][$f] > 0) {
          if ($NUM_S2[$q][$f] > 0) { ##Queries detected by both systems
	    ## Detect overlaps
	    my $OVERLAP;
	    for ($n=1;$n<=$NUM_S1[$q][$f];$n++) {
		for ($m=1;$m<=$NUM_S2[$q][$f];$m++) {
		  $OVERLAP[$n][$m] = 0;
		  $OVERLAP_S1[$n] = 0;
		  $OVERLAP_S2[$m] = 0; ## Para asegurar que un label del sistema 2 es asociada a 1 sola label del 1
	    }
	    }
	    for ($n=1;$n<=$NUM_S1[$q][$f];$n++) {
		for ($m=1;$m<=$NUM_S2[$q][$f];$m++) {
		  $dist = $CENTER_S1[$q][$f][$n] - $CENTER_S2[$q][$f][$m];
		  if ($dist<0) {$dist=-$dist;}
		  if ($dist<=$MAXDISTANCE) {
			  if ($OVERLAP_S2[$m] == 0) { ### I can only associate 1 segment of one of the files to 1 segment of the other, it is biyective
			    $OVERLAP[$n][$m] = 1;
			    $OVERLAP_S1[$n] = $m;
			    $OVERLAP_S2[$m] = $n; 
			    last; ## Cada label do sistema1 s poder overlapear com 1 label do sistema2
			  }
		  }
		}
	    }

	    
	    for ($n=1;$n<=$NUM_S1[$q][$f];$n++) {
		  if ($OVERLAP_S1[$n]!=0) { ## There is match! 
			print "$QUERY[$q] $FILE[$f] $BEGINTIME_S1[$q][$f][$n] $DURATION_S1[$q][$f][$n] 1\n";
		    }
		  else {print "$QUERY[$q] $FILE[$f] $BEGINTIME_S1[$q][$f][$n] $DURATION_S1[$q][$f][$n] 0\n";} ## There is a FA! 
	      }
	  
	    ## Print remaining candidates of system2 (not overlapped)
	    for ($m=1;$m<=$NUM_S2[$q][$f];$m++) { ## There is a MISS! 
		if ($OVERLAP_S2[$m]==0) {print "$QUERY[$q] $FILE[$f] $BEGINTIME_S2[$q][$f][$m] $DURATION_S2[$q][$f][$m] ONLY_IN_THE_REF\n";}
	    }
	  

	  }
	  else { ##Query detected only by system1
	    for ($n=1;$n<=$NUM_S1[$q][$f];$n++) { ## There is a FA!
	      print "$QUERY[$q] $FILE[$f] $BEGINTIME_S1[$q][$f][$n] $DURATION_S1[$q][$f][$n] 0\n";
	    }
	  }

    }
    elsif ($NUM_S2[$q][$f] > 0) { ##Only detected by system2
	    for ($n=1;$n<=$NUM_S2[$q][$f];$n++) { ## There is a MISS! 
	      print "$QUERY[$q] $FILE[$f] $BEGINTIME_S2[$q][$f][$n] $DURATION_S2[$q][$f][$n] ONLY_IN_THE_REF\n";
	    }
    }
  }
}

