#!/bin/bash
MATLAB_BIN="/afs/l2f/usr/local/MATLAB/R2010b/bin/matlab"
BOSARIS="/ffs/tmp/alberto/TOOLBOXES/bosaris_toolkit/"


# MediaEval 2013 - COST PARAMETERS OF THE TWV FUNCTION
P_TARGET="0.00015"
C_MISS="100"
C_FA="1"

# MediaEval 2013 Development Set: speech time (in seconds)
TSPEECH="71839"

## OUTPUT filenames
outdevscores=dev_fusion.scores 
outevalscores=eval_fusion.scores


TMP=tmpfile_$$
TMPDIR=/tmp/alberto/$$/
currentDir=`pwd`


mkdir -p $TMPDIR

function clean { rm -f $TMP.* ; rm -r $TMPDIR; }
function error { echo -e $* ; exit ; }
trap clean EXIT

if (($# != 3)) ; then
	error "\n\tSyntax: $0 dev.score evl.score stats.ref\n"
fi
if [[ $(awk '{print NF}' < $1 | uniq) != $(awk '{print NF}' < $2 | uniq) ]] ; then
	error "\nERROR: scores do not match\n"
fi

awk '{gsub("sws2013_dev_","");print}'  $1 | cut -d " " -f 1,5- > $TMPDIR/$TMP.dev 
cut -d " " -f 5- $2 > $TMPDIR/$TMP.evl 
awk '{gsub("sws2013_dev_","");print}' $3 > $TMPDIR/$TMP.stats 

cat << THEEND > $TMPDIR/$TMP.m
function $TMP
	try
		addpath(genpath('$BOSARIS'));
		S=load('$TMP.dev','-ascii');
		labels = S(:,1)';
		scores = S(:,2:end-1)';
		classf = S(:,end)';
		classf(find(classf==0)) = -1;

		R=load('$TMP.stats','-ascii');
		nTargetsPerQuery=R(:,2)';

		prior = effective_prior($P_TARGET,$C_MISS,$C_FA);
		threshold = -logit(prior);
	%%%%%%
		
		%%%%%%
		nTrials = $TSPEECH * (length(nTargetsPerQuery));

		nTargets=sum(nTargetsPerQuery);
		missingTargets=max(0,nTargets - sum(classf==1));
		missingRejections = (nTrials-nTargets)-sum(classf==-1);
		
		wscores = min(scores,[],2); % worst score for missing targets and rejections, this step is important to achieve good calibrated scores
		scoresXX = [scores , repmat(wscores,1,missingTargets), repmat(wscores,1,missingRejections)];
		classfXX = [classf , ones(1,missingTargets), -ones(1,missingRejections)];

		[f,nxe_train,w] = train_linear_fuser(scoresXX,classfXX,prior,true,true,1000,[]);
	
		[~,~,w2] = train_linear_fuser(f(scoresXX)',classfXX,prior,true,true,1000,[]);
		fprintf(1,'check 1 0 --> %g %g\n',w2(1),w2(2));


		
		scale = w(1:end-1)
		offset = w(end)
		save('$TMP.fuse_params','-ascii','w');
		estimated_atwv = ATWV(f(scores)',labels,classf,prior,$TSPEECH,nTargetsPerQuery)

		%% apply fusion/calibration to the dev scores
		scores = f(S(:,2:end-1)'); 
		dec=scores>threshold;
		scores = [scores dec];
		save('$TMP.dev.fused','-ascii','scores');

		%% apply fusion/calibration to the eval scores
		S=load('$TMP.evl','-ascii'); 
		scores = f(S(:,1:end-1)'); 
		dec=scores>threshold;
		scores = [scores dec];
		save('$TMP.evl.fused','-ascii','scores');
	catch
		fprintf('%s\n',lasterr)
	end
	quit force

	function [v,pmiss,pfa] = ATWV(scores,labels,classf,prior,totalspeech, nTargetsPerQuery, threshold)
	% Computes the Average Term-Weighted Value  
	% Inputs:
	%   scores: A vector of scores 
	%   labels: A vector containing term-identity labels
	%   classf: A vector containing +1/-1 to indicate target/non-target trials
	%   prior: The effective target prior.
	%   threshold: The threshold to be used. If [], -logit(prior) is used.
	% Outputs:
	%   c: the Average Term-Weighted Value
	
		if ~exist('threshold','var') || isempty(threshold)
			threshold = -logit(prior)
		end
		beta = (1 - prior) / prior;
	
		tar = classf == 1;
		ntar = classf == -1;
		pmiss = zeros(1,length(nTargetsPerQuery));
		pfa = pmiss;
		for i = 1:length(nTargetsPerQuery)

			if (isempty(labels==i)), %% This query was not detected
			  pmiss(i)=1;
			  pfa(i)=0;
			else
			  x = labels==i & tar;
			  nTargets = nTargetsPerQuery(i);
			  if (nTargets>0)
			    pdet = sum(x & scores>threshold)/nTargets;
			  else
			    pdet = 1;
			  end
			  pmiss(i) = 1 - pdet;
			  x = labels==i & ntar;
			  %pfa(i) = sum(x & scores>threshold)/sum(x);
			  pfa(i) = sum(x & scores>threshold)/(totalspeech-nTargets);
			end
		end
		v = 1 - mean(pmiss + beta * pfa);
	end

end
THEEND

cd $TMPDIR
$MATLAB_BIN -nosplash -nodisplay -nojvm -r $TMP 1>&2 < /dev/null
awk '{printf("%.6f %d\n",$1,$2)}' $TMP.evl.fused  > $TMP.evl.fused2
awk '{printf("%.6f %d\n",$1,$2)}' $TMP.dev.fused  > $TMP.dev.fused2
cp $TMP.fuse_params $currentDir/fuse_params.txt
cd $currentDir

awk '{print $1,$2,$3,$4}' $1 | paste -d " " - $TMPDIR/$TMP.dev.fused2 > $outdevscores
awk '{print $1,$2,$3,$4}' $2 | paste -d " " - $TMPDIR/$TMP.evl.fused2 > $outevalscores

