#!/usr/bin/perl 

############################################################################################################################
#                                                                                                                          #
#                                                  INESC-ID/L2F                                                            #
#                               Instituto de Engenharia de Sistemas e Computadores                                         #
#                                    Investigaco e Desenvolvimento em Lisboa,                                              #
#                                 Laboratario de Sistemas de Lingua Falada (L2F)                                           #
#                                                                                                                          #
#                                                                                                                          #
############################################################################################################################
#                                                                                                                          #
#                                                                                                                          #
# Authors: Alberto Abad     (alberto@l2f.inesc-id.pt)                                                                      #
#                                                                                                                          #
# Date   : 11 Oct 2013                                                                                                     #
#                                                                                                                          #
# Version: 1.0                                                                                                             #
#                                                                                                                          #
############################################################################################################################
#
#  This perl script is used by the PrepareForFusion.sh script to apply heurestic rules to hypothesize the missing scores of
#  the systems. 
#  
#  It takes as input the aligned detection candidate scores, the number of systems, a flag indicating the type of heurestic
#  method to apply and a list of all queries                                                                               
#
#  There are 4 implemented methods for herestic score hypothesizing: mean of the remaining actual detected scores, minimum 
#  score given by a specifc system for that missing query, global minimum score given by a specific system, and histogram 
#  based method taht tries to find the missing score equivalent in terms of "histogram position" to the ones actually given
#  by the remaining systems (this idea never really worked very well)
#
#     
###########################################################################################################################
my $nArgs = $#ARGV +1;

if ($nArgs!=4) {die "Invalid number of arguments $nArgs";}

my $scoreFile = $ARGV[0];
my $nSys = $ARGV[1];
my $heuristicScore = $ARGV[2];
my $querylist = $ARGV[3];



# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# 4 MODES OF HEURISTICS SCORES
#   0: MEAN OF THE ACTUAL DETECTED SCORES
#   1: MINIMUM PER QUERY
#   2: GLOBAL MINIMUM OF THE SYSTEMS
#   3: HISTOGRAM BASED METHOD
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# 0: MEAN OF THE ACTUAL DETECTED SCORES
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
if ($heuristicScore==0) {

  ## Print the output scores for this mode
  open SCORES, $scoreFile or die "could not open score file $scoreFile";
  while (<SCORES>) {
    chomp;

      @line = split(/ /, $_);

      $query = $line[0];
      $file = $line[1];
      $bt = $line[2];
      $dur = $line[3];

      shift(@line);
      shift(@line);
      shift(@line);
      shift(@line);
      
      print "$query $file $bt $dur ";
      ## Compute mean for this line
      $ndet=0;
      $score=0;
      for ($i=0;$i<$nSys; $i++)
      {
	if (! ($line[$i] eq "NOT_DETECTED")) {
	  $score=$score + $line[$i];
	  $ndet++;
	 }
      }

      $score=$score/$ndet;

      ## Print scores for this line replacing by mean the not detected ones
      for ($i=0;$i<$nSys; $i++)
      {
	if (! ($line[$i] eq "NOT_DETECTED"))  { printf("%.4f",$line[$i]);}
	else { printf("%.4f",$score);}

	if ($i==($nSys-1))  {print "\n";}
	else {print " ";}
      }
  }

  close(SCORES);

}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#   1:2: MINIMUM PER QUERY AND GLOBAL MINIMUM
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
elsif (($heuristicScore==1)||($heuristicScore==2)) { 


  my %mapq = ();                                                                                                                                                                                                                                                                  

  ## Read list of queries
																																
  open QUERIES, $querylist or die "could not open query list file $querylist";                            
  $value=1;                                                                                                                                                                              
  while (<QUERIES>) {                                                                                                                                                                                                                                                            
  chomp;                                                                                                                                                                                                                                                                         
  $label=$_;
  $mapq{$label} = $value;
  $max_num_queries = $value;
  $value=$value+1;
  }
  close(QUERIES);


  
 for ($i=0;$i<$nSys; $i++) {
  $MIN_SCORE[$i]=1000000000;
  for ($q=1;$q<=$max_num_queries;$q++) {
    $DEF_SCORE[$i][$q]=1000000000;
    $DEF_SCORE[$i][$q]=1000000000;
  }  
 }

  ## READ AND STORE THE SCORES OF THE SYSTEMS
  open SCORES, $scoreFile or die "could not open score file $scoreFile";
  while (<SCORES>) {
    chomp;

      @line = split(/ /, $_);

	$query=$line[0];

	shift(@line);
	shift(@line);
	shift(@line);
	shift(@line);
	  
      for ($i=0;$i<$nSys; $i++)
      {
	  $sc=$line[$i];
	  if (!($sc eq "NOT_DETECTED")) {
	      if ($sc<$DEF_SCORE[$i][$mapq{$query}]) {$DEF_SCORE[$i][$mapq{$query}]=$sc;} #minimum per query
	      if ($sc<$MIN_SCORE[$i]) {$MIN_SCORE[$i]=$sc;} #minimum global
	  }
      }
  }

  close(SCORES);


    if ($heuristicScore==1) {## Consider a minimum per query
      for ($i=0;$i<$nSys; $i++) {
	  for ($q=1;$q<=$max_num_queries;$q++) { # If the query never appeared, assign the global value
	      if ($DEF_SCORE[$i][$q]==1000000000) {$DEF_SCORE[$i][$q]=$MIN_SCORE[$i];} 
	  }
      }  
  
    }
    else { ## Consider a global minimum
     for ($i=0;$i<$nSys; $i++) {
	  for ($q=1;$q<=$max_num_queries;$q++) { # If the query never appeared, assign the global value
	      $DEF_SCORE[$i][$q]=$MIN_SCORE[$i];
	  }
      }  
    }

 open SCORES, $scoreFile or die "could not open score file $scoreFile";
  while (<SCORES>) {
    chomp;

      @line = split(/ /, $_);

	$query=$line[0];

	$query = $line[0];
	$file = $line[1];
	$bt = $line[2];
	$dur = $line[3];

	shift(@line);
	shift(@line);
	shift(@line);
	shift(@line);
	
	print "$query $file $bt $dur ";
	  
      for ($i=0;$i<$nSys; $i++)
      {
	  if (!($line[$i] eq "NOT_DETECTED")) { printf("%.4f",$line[$i]);}
	  else { printf("%.4f",$DEF_SCORE[$i][$mapq{$query}]);}

	 if ($i==($nSys-1))  {print "\n";}
	 else {print " ";}
      }
  }

  close(SCORES);

}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#   3: HISTOGRAM BASED METHOD
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
elsif ($heuristicScore==3) {


  for ($i=0;$i<$nSys; $i++) {$numScores[$i]=0;}

  ## READ AND STORE THE SCORES OF THE SYSTEMS
  open SCORES, $scoreFile or die "could not open score file $scoreFile";
  while (<SCORES>) {
    chomp;

      @line = split(/ /, $_);

	shift(@line);
	shift(@line);
	shift(@line);
	shift(@line);
	  
      for ($i=0;$i<$nSys; $i++)
      {
	if (!($line[$i] eq "NOT_DETECTED")) {$SC[$i][$numScores[$i]++]=$line[$i];}
      }
  }
  close(SCORES);

  # #   Compute histograms
  for ($i=0;$i<$nSys; $i++) {

    ## Prepare for histogram type of default score
    @{$SC_SORTED[$i]}= sort {$a <=> $b} @{$SC[$i]};
  
    @sort_val=@{$SC_SORTED[$i]};
    $numScores[$i] = $#sort_val;

    for ($n=0;$n<=$#sort_val;$n++) {
      $histmap[$i]{$SC_SORTED[$i][$n]} = $n;
    }

  }


  ## Print the output scores for this mode
  open SCORES, $scoreFile or die "could not open score file $scoreFile";
  while (<SCORES>) {
    chomp;

      @line = split(/ /, $_);

     	$query=$line[0];

	$query = $line[0];
	$file = $line[1];
	$bt = $line[2];
	$dur = $line[3];

	shift(@line);
	shift(@line);
	shift(@line);
	shift(@line);
	
	print "$query $file $bt $dur ";

      for ($i=0;$i<$nSys; $i++)
      {
	$ndet=0;
	$score=0;

	if ($line[$i] eq "NOT_DETECTED") {
	  for ($j=0;$j<$nSys; $j++) {

	    if (! ($line[$j] eq "NOT_DETECTED")) {
		  $index_ori=$histmap[$j]{$line[$j]};
		  $index_dest=int($numScores[$i]*$index_ori/$numScores[$j]);
		  $score=$score + $SC_SORTED[$i][$index_dest];
		  $ndet++;
	    }
	  }
	  $score=$score/$ndet;
	}
	else {$score = $line[$i];}
	
	if ($i==0) { printf("%.4f",$score);}
	else {printf(" %.4f",$score);}
      }

      print "\n";
  }

  close(SCORES);

}

